package cn.iocoder.yudao.module.system.service.robot;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.RobotWarnMsgPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.RobotWarnMsgSaveReqVO;
import cn.iocoder.yudao.module.system.controller.admin.statistics.dto.RobotWarnMsgClassificationDTO;
import cn.iocoder.yudao.module.system.controller.admin.statistics.dto.RobotWorkHourStatisticsDTO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotWarnMsgDO;
import com.baomidou.mybatisplus.extension.service.IService;

/**
 * 机器人告警信息 Service 接口
 *
 * @author 陈宾顺
 */
public interface RobotWarnMsgService extends IService<RobotWarnMsgDO> {

    /**
     * 创建机器人告警信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createWarnMsg(@Valid RobotWarnMsgSaveReqVO createReqVO);

    /**
     * 更新机器人告警信息
     *
     * @param updateReqVO 更新信息
     */
    void updateWarnMsg(@Valid RobotWarnMsgSaveReqVO updateReqVO);

    /**
     * 删除机器人告警信息
     *
     * @param id 编号
     */
    void deleteWarnMsg(Long id);

    /**
     * 获得机器人告警信息
     *
     * @param id 编号
     * @return 机器人告警信息
     */
    RobotWarnMsgDO getWarnMsg(Long id);

    /**
     * 获得机器人告警信息分页
     *
     * @param pageReqVO 分页查询
     * @return 机器人告警信息分页
     */
    PageResult<RobotWarnMsgDO> getWarnMsgPage(RobotWarnMsgPageReqVO pageReqVO);

    void sendWarnMsgToWebsocket(Object errorMsg);

    /**
     * 添加告警信息
     * @param message
     */
    void addWarnMsg(String message);

    List<RobotWarnMsgDO> getReadPage();

    String allRead();

    List<RobotWarnMsgDO> getWarnMsgByDetailIds(List<Long> detailIds);

    /**
     * 统计故障根因分析   type  1:周, 2:月,  3:季度
     * @param type
     * @return
     */
    Map<String,List<RobotWarnMsgClassificationDTO>> robotWarnMsgClassification(String type);
}
